module.exports = {
  name: 'healer',
  prio: 10,
  tickInit () {
    for (let room of Object.values(Game.rooms)) {
      if (!room.memory.healer) {
        room.memory.healer = {}
      }
      if (room.find(FIND_STRUCTURES, {filter: s => s.structureType === STRUCTURE_TOWER}).length > 0) {
        room.memory.healer.targetCount = 0
        continue
      }
      room.memory.healer.targetCount = Math.min(room.find(FIND_HOSTILE_CREEPS).length, 1)
    }
  },
  nextSpawn (spawn, roleCreeps) {
    for (let room of Object.values(Game.rooms)) {
      if (_.filter(roleCreeps, {memory: {room: room.name}}).length < room.memory.healer.targetCount) {
        return [
          [MOVE, MOVE, MOVE, MOVE, HEAL, HEAL, HEAL, HEAL],
          {room: room.name}
        ]
      }
    }
  },
  tick (creep) {
    let room = Game.rooms[creep.memory.room]
    let targetRoomName = creep.memory.room
    if (creep.memory.room !== creep.pos.roomName
        && (_.filter(Game.creeps, {memory: {role: 'healer', room: targetRoomName}, spawning: false}).length < Memory.rooms[targetRoomName].healer.targetCount
            || _.filter(Game.creeps, {memory: {role: 'attacker', room: targetRoomName}, spawning: false}).length < Memory.rooms[targetRoomName].attacker.targetCount
            || _.filter(Game.creeps, {memory: {role: 'rangedAttacker', room: targetRoomName}, spawning: false}).length < Memory.rooms[targetRoomName].rangedAttacker.targetCount)
    ) {
      creep.moveTo(creep.pos.findClosestByPath(FIND_FLAGS, {filter: el => el.name.startsWith('gather'), visualizePathStyle: {stroke: '#0017ff'}}))
      return
    }
    if (!room || creep.memory.room !== creep.pos.roomName) {
      creep.moveTo(new RoomPosition(25, 25, creep.memory.room))
      return
    }
    if (creep.pos.x <= 2) {
      creep.move(RIGHT)
    } else if (creep.pos.y <= 2) {
      creep.move(BOTTOM)
    } else if (creep.pos.x >= 47) {
      creep.move(LEFT)
    } else if (creep.pos.y >= 47) {
      creep.move(TOP)
    }

    let targetCreep
    if (creep.memory.lastTarget) {
      targetCreep = Game.getObjectById(creep.memory.lastTarget)
    }

    let closeAttackers = creep.pos.findInRange(FIND_HOSTILE_CREEPS, 4, {filter: c => c.getActiveBodyparts(ATTACK) > 0 && c.pos.roomName === targetRoomName})
    if (closeAttackers.length > 0) {
      creep.moveByPath(PathFinder.search(creep.pos, closeAttackers.map(el => {return {pos: el.pos, range: 3}}), {flee: true, maxRooms: 1}).path)
    }

    if (creep.hits < creep.hitsMax) {
      creep.heal(creep)
      return
    }

    let healTargets = creep.pos.findInRange(FIND_MY_CREEPS, 1, {filter: c => c.hits < c.hitsMax && c.pos.roomName === targetRoomName})
    if (healTargets.length > 0) {
      creep.heal(healTargets[0])
      return
    }

    let rangedHealTarget = creep.pos.findClosestByRange(FIND_MY_CREEPS, {filter: c => c.hits < c.hitsMax && c.pos.roomName === targetRoomName})
    if (rangedHealTarget) {
      creep.rangedHeal(rangedHealTarget)
      creep.moveTo(rangedHealTarget, {visualizePathStyle: {stroke: '#00ff0f'}})
      return
    }

    let possibleTarget = creep.pos.findClosestByPath(FIND_MY_CREEPS, {filter: c => creepIsAttacker(c) && c.pos.roomName === targetRoomName})
    let range = creep.pos.getRangeTo(possibleTarget)
    if (range === 1) {
      creep.moveByPath(PathFinder.search(creep.pos, {pos: possibleTarget.pos, range: 2}, {flee: true, maxRooms: 1}).path)
    }
    else if (range > 3) {
      creep.moveTo(possibleTarget, {visualizePathStyle: {stroke: '#00ff0f'}})
    }
  },
}
